// Set up namespace and some state.
var mkdru = {
  // Settings to pass to pz2.js
  usesessions: Drupal.settings.mkdru.use_sessions === '1',
  //   showResponseType: 'json',
  // Variables
  submitted: false,
  pz2: null,
  totalRec: 0,
  pagerRange: 6,
  sourceMax: Drupal.settings.mkdru.source_max,
  subjectMax: Drupal.settings.mkdru.subject_max,
  authorMax: Drupal.settings.mkdru.author_max,
  pazpar2Path: Drupal.settings.mkdru.pz2_path,
  // State
  defaultState: {
    page: 1,
    perpage: 20,
    sort: 'relevance',
    filter: null,
    query:null,
    recid:null
  },
  state: {}
};



// So we can use jQuery BBQ with Drupal 6 and its 1.2.6 jQuery
if (!$.isArray) $.isArray = function(obj) {
  return Object.prototype.toString.call(obj) === "[object Array]";
},



// pz2.js event handlers:
mkdru.pz2Init = function () {
  if (mkdru.state.query) {
    mkdru.search();
  }
  //mkdru.pz2.stat();
  //mkdru.pz2.bytarget();
};

mkdru.pz2Show = function (data) {
  mkdru.totalRec = data.merged;
  $('.mkdru-pager').html(mkdru.generatePager());
  var countsHtml = Drupal.t('Displaying: ') + (data.start + 1) + 
    Drupal.t(' to ') + (data.start + data.num) + Drupal.t(' of ')
    + data.merged + Drupal.t(' (found: ') + data.total + ')';
  $('.mkdru-counts').html(countsHtml);

  var html = "";
  for (var i = 0; i < data.hits.length; i++) {
    html += Drupal.theme('mkdruResult', data.hits[i], 
      i + 1 + mkdru.state.perpage * (mkdru.state.page - 1),
      "#" + $.param.fragment($.param.fragment(
        window.location.href, {recid: data.hits[i].recid})) + "\n"
    );
  }
  $('.mkdru-result-list').html(html);
  if (mkdru.state.recid) {
    mkdru.pz2.record(mkdru.state.recid);
  }
  else {
    $('.mkdru-results').show();
  }
};

mkdru.pz2Status = function (data) {
};

mkdru.pz2Term = function (data) {
  var html = "";
  for (var i = 0; i < data.xtargets.length && i < mkdru.sourceMax; i++ ) {
    html += Drupal.theme('mkdruTerm', data.xtargets[i].name, data.xtargets[i].freq, 
                          'mkdru-facet-source', data.xtargets[i].id);
  }
  $('.mkdru-facet-sources').html(html);

  html = "";
  for (var i = 0; i < data.subject.length && i < mkdru.subjectMax; i++ ) {
    html += Drupal.theme('mkdruTerm', data.subject[i].name, data.subject[i].freq, 
                          'mkdru-facet-subject', data.subject[i].id);
  }
  $('.mkdru-facet-subjects').html(html);

  html = "";
  for (var i = 0; i < data.author.length && i < mkdru.authorMax; i++ ) {
    html += Drupal.theme('mkdruTerm', data.author[i].name, data.author[i].freq, 
                          'mkdru-facet-author', data.author[i].id);
  }
  $('.mkdru-facet-authors').html(html);

  $('.mkdru-facet-source').bind('click', function (e) {
    mkdru.limitTarget(this.getAttribute('target_id'), this.firstChild.nodeValue);
    return false;
  });

  $('.mkdru-facet-subject').bind('click', function (e) {
    mkdru.limitQuery('su', this.firstChild.nodeValue);
    return false;
  });

  $('.mkdru-facet-author').bind('click', function (e) {
    mkdru.limitQuery('au', this.firstChild.nodeValue);
    return false;
  });
};

mkdru.pz2ByTarget = function (data) {
  
};

mkdru.pz2Record = function (data) {
  clearTimeout(mkdru.pz2.showTimer);
  $('.mkdru-results').hide();
  $('.mkdru-detail').html(Drupal.theme('mkdruDetail', data));
  $('.mkdru-detail-back').bind('click', function () {$.bbq.removeState('recid');});
  $('.mkdru-detail').show();
  clearTimeout(mkdru.pz2.recordTimer);
};



// State and URL handling 

// populate state from an object and fill in the blanks with defaults
mkdru.stateFromObject = function (obj) {
  for (var key in mkdru.defaultState) {
    if (typeof(obj[key]) != "undefined") {
      mkdru.state[key] = obj[key];
    }
    else {
      mkdru.state[key] = mkdru.defaultState[key];
    }
  }
};

// populate state from current window's hash string
mkdru.stateFromHash = function () {
  mkdru.stateFromObject($.deparam.fragment());
};

// set current window's hash string from state
mkdru.hashFromState = function () {
  // only include non-default settings in the URL
  var alteredState = {};
  for (var key in mkdru.defaultState) {
    if (mkdru.state[key] != mkdru.defaultState[key]) {
      alteredState[key] = mkdru.state[key];
    }
  }
  $.bbq.pushState(alteredState);
};

// update mkdru_form theme's ui to match state
mkdru.uiFromState = function () {
  for (var key in mkdru.state) {
    switch(key) {
    case 'query':
      $('.mkdru-search input:text').attr('value', mkdru.state[key]);
      break;
    case 'perpage':
      $('.mkdru-perpage').attr('value', mkdru.state[key]);
      break;
    case 'sort':
      $('.mkdru-sort').attr('value', mkdru.state[key]);
      break;
    }
  }
};

mkdru.hashChange = function () {
  mkdru.stateFromHash();
  // Request for details
  if (mkdru.state.recid) {
    mkdru.pz2.record(mkdru.state.recid);
  }
  // Other internal link
  else {
    mkdru.pz2.showPage(mkdru.state.page-1);
    $('.mkdru-detail').hide();
    $('.mkdru-results').show();
  }
};



//search bo handler ONLY
mkdru.submitQuery = function () {
  mkdru.state.query = $('.mkdru-search input:text').attr('value');
  mkdru.pollDropDowns();
  mkdru.resetPage();
  mkdru.hashFromState();
  mkdru.search();
  mkdru.submitted = true;
  return false;
};

//criteria drop-downs (perpage,sort) handler ONLY
mkdru.submitCriteria = function () {
  mkdru.pollDropDowns();
  //search is not ON, do nothing
  if (!mkdru.submitted) return false;
  mkdru.resetPage();
  mkdru.hashFromState();
  mkdru.pz2.show(0, mkdru.state.perpage, mkdru.state.sort);
  return false;
}

mkdru.search = function () {
  mkdru.pz2.search(mkdru.state.query, mkdru.state.perpage, mkdru.state.sort,
      mkdru.state.filter);
  //inform others that the search is ON
  mkdru.submitted = true;
};

mkdru.pollDropDowns = function () {
  mkdru.state.perpage = $('.mkdru-perpage').attr('value');
  mkdru.state.sort = $('.mkdru-sort').attr('value');
};

//TODO it's rude to mess with user's query, we should have a breadcrumb
mkdru.limitQuery = function (field, value) {
  $('.mkdru-search input:text').attr('value', function () {
    return this.value += ' and ' + field + '="' + value + '"';
  });
  mkdru.submitQuery();
};

mkdru.limitTarget = function (id, name) {
  mkdru.state.filter = 'pz:id=' + id;
  mkdru.pollDropDowns();
  mkdru.resetPage();
  mkdru.hashFromState();
  mkdru.search();
  var navi = document.getElementById('mkdru-navi');
  if (!navi) return false;
  navi.innerHTML = 
        'Source: <a class="crossout" href="#" onclick="delimitTarget();return false;">'
        + name + '</a>';
  navi.innerHTML += '<hr/>';
  return false;
};

mkdru.generatePager = function () {
 // make sure page param is a number, otherwise pageing frwd will be broken
  if (typeof mkdru.state.page == "string") {
    mkdru.state.page = Number(mkdru.state.page);
  }
  var total = Math.ceil(mkdru.totalRec / mkdru.state.perpage);
  var first = (mkdru.state.page - mkdru.pagerRange > 0)
      ? mkdru.state.page - mkdru.pagerRange : 1;
  var last = first + 2 * mkdru.pagerRange < total
      ? first + 2 * mkdru.pagerRange : total;  
  var prev = null;
  var next = null;
  var pages = [];

  if ((mkdru.state.page - 1) >= first) {
    prev = "#" + $.param.fragment($.param.fragment(
               window.location.href, {page: mkdru.state.page - 1}))
  }
  if ((mkdru.state.page + 1) <= total) {
    next = "#" + $.param.fragment($.param.fragment(
               window.location.href, {page: mkdru.state.page + 1}))
  }

  for (var i = first; i <= last; i++) {
    pages.push("#" + $.param.fragment($.param.fragment(
               window.location.href, {page: i})));
  }

  return Drupal.theme('mkdruPager', pages, first, mkdru.state.page,
                      total, prev, next);
};

mkdru.resetPage = function () {
  mkdru.state.page = 1;
  mkdru.totalRec = 0;
};



// wait until the DOM is ready, bind events
// and instantiate pz2 library
$(document).ready(function () {
  $(window).bind( 'hashchange', mkdru.hashChange);
  $('.mkdru-search').bind('submit', mkdru.submitQuery);
  $('.mkdru-search input:text').attr('value', '');
  $('.mkdru-perpage').bind('change', mkdru.submitCriteria);
  $('.mkdru-sort').bind('change', mkdru.submitCriteria);

  mkdru.pz2 = new pz2( { "onshow": mkdru.pz2Show,
              "showtime": 500, //each timer (show, stat, term, bytarget) can be specified this way
              "pazpar2path": mkdru.pazpar2path,
              "oninit": mkdru.pz2Init,
              "onstat": mkdru.pz2Status,
              "onterm": mkdru.pz2Term,
              "termlist": "xtargets,subject,author",
              "onbytarget": mkdru.pz2ByTarget,
              "usesessions" : mkdru.usesessions,
              "showResponseType": mkdru.showResponseType,
              "onrecord": mkdru.pz2Record,
              "autoInit": false } );

  // initialise state to hash string or defaults
  mkdru.stateFromHash();
  // and update UI to match
  mkdru.uiFromState();

  // ting thing
  if (typeof(Drupal.settings.mkdru.query) !== "undefined") {
    mkdru.state.query = Drupal.settings.mkdru.query
  }

  if (mkdru.usesessions) {
    mkdru.pz2.init();
  }
  else if (mkdru.state.recid) {
    mkdru.pz2.record(mkdru.state.recid);
  }
  else if (mkdru.state.query) {
    mkdru.search();
  }
});
